'use strict';

var defaultTemplate = (function (tpl, data) {
  if (!data) { return tpl; }

  var re = /{(.*?)}/g;

  return tpl.replace(re, function (_, key) {
    var ret = data;

    for (var prop of key.split('.')) {
      ret = ret ? ret[prop] : '';
    }

    return ret || '';
  });
});

function getPathSegments(path) {
  var pathArr = path.split('.');
  var parts = [];

  for (var i = 0; i < pathArr.length; i++) {
    var p = pathArr[i];

    while (p[p.length - 1] === '\\' && pathArr[i + 1] !== undefined) {
      p = p.slice(0, -1) + '.';
      p += pathArr[++i];
    }

    parts.push(p);
  }

  return parts;
}

function getProp (data, path) {
  return getPathSegments(path).reduce(function (obj, k) { return obj && obj[k]; }, data);
}

var Vue;

var Inter = function Inter(ref) {
  var locale = ref.locale;
  var locales = ref.locales; if ( locales === void 0 ) locales = {};
  var template = ref.template; if ( template === void 0 ) template = defaultTemplate;

  if (process.env.NODE_ENV === 'development' && !Vue) {
    throw new Error('You have to install `vue-inter` first: Vue.use(Inter)');
  }

  this.template = template;
  this.locales = locales;

  Vue.util.defineReactive(this, '__locale', locale);
};

var prototypeAccessors = { currentLocale: { configurable: true },availableLocales: { configurable: true } };

Inter.install = function install (_Vue) {
  Vue = _Vue;
  Vue.mixin({
    beforeCreate: function beforeCreate() {
      this.$inter = this.$options.inter || this.$parent && this.$parent.$inter;
      if (this.$inter) {
        this.$i = this.$inter.get.bind(this.$inter);
      }
    }
  });
};

Inter.prototype.get = function get (path) {
    var data = [], len = arguments.length - 1;
    while ( len-- > 0 ) data[ len ] = arguments[ len + 1 ];

  var localeData = this.locales[this.currentLocale];
  if (process.env.NODE_ENV === 'development' && !localeData) {
    throw new Error(("[vue-inter] Locale \"" + (this.currentLocale) + "\" was not found"));
  }
  var message = getProp(localeData, path);
  if (process.env.NODE_ENV === 'development' && !message) {
    throw new Error(("[vue-inter] No message under \"" + path + "\" was found"));
  }
  if (typeof message === 'function') {
    return message.apply(void 0, data);
  }
  return (ref = this).template.apply(ref, [ message ].concat( data ));
    var ref;
};

prototypeAccessors.currentLocale.get = function () {
  return this.__locale;
};

Inter.prototype.setCurrentLocale = function setCurrentLocale (locale) {
  this.__locale = locale;
  return this;
};

Inter.prototype.setLocaleData = function setLocaleData (locale, localData) {
  this.locales[locale] = localData;
  return this;
};

prototypeAccessors.availableLocales.get = function () {
  return Object.keys(this.locales);
};

Object.defineProperties( Inter.prototype, prototypeAccessors );

module.exports = Inter;
